****************************************************************************
****************************************************************************
**                                                               
**  Monster - Data Structure Definitons
**
**  by Patrick Davidson (eeulplek@hotmail.com, http://www.ocf.berkeley.edu/~pad)
**
**  Last updated March 17, 2016
**
****************************************************************************
****************************************************************************

        IFD     ti89
WIDTH_BYTES     set     20
IS89            set     1
TILE_HEIGHT     set     8
PLANE_LINES     set     96
PLANE_SIZE      set     1920
        ENDIF

        IFND    ti89
WIDTH_BYTES     set     30
IS89            set     0
TILE_HEIGHT     set     10
PLANE_LINES     set     120
PLANE_SIZE      set     3600
        ENDIF

******************************************** BALL DATA STRUCTURE

b_type  equ     0       ; 0 - non-existent, 1 - cought, 2 - normal
b_speed equ     2       ; velocity multiplied by this
b_x     equ     4       ; all coords shifted left 6 bits
b_y     equ     6                                           
b_angle equ     8       ; angle (0 steepest, 16 shallowest)
b_xdir  equ     10      ; sign of X-direction
b_ydir  equ     12      ; sign of Y-direction
b_oldx  equ     14      ; previous coordinates (for trails)
b_oldy  equ     16
b_voldx equ     18
b_voldy equ     20
b_time  equ     22      ; counter to accelerate periodicallyd
b_size  equ     24
b_num   equ     3

******************************************** LASER DATA STRUCTURE

l_y     equ     0
l_x     equ     2
l_size  equ     4
l_num   equ     16

******************************************** MAIN VARIABLES
*
* This defines the game's main variables.  The 'rs' macro reserves the
* specified amount of space by incrementing the offset counter.  Overall,
* this section serves to calculate the offsets of each variable.  This
* system allows variables to be easily added, removed, or resized, as the
* offsets themselves are not hard-coded.  Near the bottom, data_end is
* defined with a size of zero to set that equal to the length of the data
* so far.  At the end, 'offset' is the total size of all this data.
*
********

offset  set     0       

rs      MACRO   
\1      set     offset  
offset  set     offset+\2       
        ENDM    

        rs      s,PLANE_SIZE            ; Screen buffer
        rs      s2,PLANE_SIZE
        rs      temp_buffer,20
        rs      _string_buffer,40
        rs      plane0,4
        rs      plane1,4
        rs      key_data,10
        rs      old_keys,10
        rs      key_edge,10
        rs      screenx,2
        rs      bounce_x,2
        rs      bounce_y,2
        rs      bounce_status,2
        rs      bounce_xv,1
        rs      bounce_yv,1
        rs      exit_status,2           ; 0=exit, 1=hs/exit, -1=hs/return
        rs      level_pointer,4
        rs      num_levels,2

        rs      data_start,0            ; Start of data to be saved

        rs      speed,2
        rs      speed_bonus,4
        rs      victory_bonus,4

        rs      data_clear_start,0

        rs      pausedelay,2
        rs      paused,2
        rs      cheat,2
        rs      score,4
        rs      score_inc,4
        rs      bonus_type,2
        rs      bonus_x,2
        rs      bonus_y,2
        rs      temp_x,2
        rs      temp_y,2
        rs      game_timer,2
        rs      brickthru,2
        rs      saved_game,2
        rs      num_balls,2
        rs      paddlex,2
        rs      paddlew,2
        rs      lives,2
        rs      level,2
        rs      bricks_left,2
        rs      balls,b_num*b_size
        rs      s0,20
        rs      map,18*12
        rs      s1,20
        rs      level_name,20
        rs      laserpower,2
        rs      laserblasts,l_num*l_size
        rs      seed,2
        
        rs      data_end,0              ; End of data to be saved

*************************************** EXIT STATUS INDICATORS

EXIT_NORMAL     equ     0
EXIT_HIGHSCORE  equ     1
EXIT_SHOWSCORE  equ     -1
EXIT_QUIT       equ     10
EXIT_NOLEVEL    equ     11

*************************************** HARDWARE KEY READING
*
* This macro tests a key, using data from the last call to Read_All_Keys.
* Its syntax is
*
* GETKEY row89,column89,row92,column92
*
* The zero flag is set if it is pressed, and clear otherwise.
*
* A5 must be pointing to the game's data structures.
*
********

GETKEY  MACRO
    IFD ti89
        btst    #\2,key_data+\1(a5)
    ENDIF
    IFND ti89
        btst    #\4,key_data+\3(a5)
    ENDIF
        ENDM

GETEDGE  MACRO
    IFD ti89
        btst    #\2,key_edge+\1(a5)
    ENDIF
    IFND ti89
        btst    #\4,key_edge+\3(a5)
    ENDIF
        ENDM

*************************************** ROM CALLING FUNCTIONS

    IFND ti92
JMP_ROM MACRO
        move.l  (200).w,a0
        move.l  \1*4(a0),a0
        jmp     (a0)
        ENDM

JSR_ROM MACRO
        move.l  (200).w,a0
        move.l  \1*4(a0),a0
        jsr     (a0)
        ENDM
    ENDIF

    IFD ti92
tios::DrawStr equ tios::DrawStrXY
tios::HeapAllocHigh equ tios::HeapAlloc

JMP_ROM MACRO
        jmp     tios::\1
        ENDM

JSR_ROM MACRO
        jsr     tios::\1
        ENDM
    ENDIF